@@ -2,6 +2,7 @@ import logging
 import pandas as pd
 import numpy as np
 from lightgbm import LGBMClassifier
+from sklearn.base import BaseEstimator
 from sklearn.exceptions import NotFittedError
 from sklearn.utils.validation import check_is_fitted
 DEFAULT_MODEL = LGBMClassifier
@@ -11,7 +12,7 @@ class AdHocBoost:
 
     def __init__(self, number_stages=2, stage_model_cls=DEFAULT_MODEL,
                  stage_model_hyper_param_sets=None, stage_proba_thresholds=None,
-                 sample_weight_multipliers=None):
+                 stage_positive_sample_weights=None):
         """
         Init function.
         :param number_stages: The number of stages in the model.
@@ -29,8 +30,8 @@ class AdHocBoost:
         # Load default parameters if necessary.
         if stage_proba_thresholds is None:
             stage_proba_thresholds = [1e-3]
-        if sample_weight_multipliers is None:
-            sample_weight_multipliers = [1.7, 1.7]
+        if stage_positive_sample_weights is None:
+            stage_positive_sample_weights = [1.7, 1.7]
         if stage_model_hyper_param_sets is None:
             stage_model_hyper_param_sets = [
                 {'learning_rate': 0.1762, 'min_data_in_leaf': 17, 'num_leaves': 29},
@@ -40,24 +41,32 @@ class AdHocBoost:
         try:
             assert number_stages == len(stage_model_hyper_param_sets) and \
                    number_stages - 1 == len(stage_proba_thresholds) and \
-                   number_stages == len(sample_weight_multipliers)
+                   number_stages == len(stage_positive_sample_weights)
         except AssertionError as error:
             logging.error(f"{error}: Check number_stages, stage_model_hyper_param_sets, and stage_proba_thresholds; "
                           f"they disagree about number of stages in the model.")
             raise error
 
-        # AdHocBoost uses some built-in sklearn functionality, so it needs to ascertain
+        # AdHocBoost uses some built-in sklearn functionality, so it needs to ensure that the base model class being
+        # used inherits sklearn BaseEstimator
+        try:
+            assert issubclass(stage_model_cls, BaseEstimator)
+        except AssertionError as e:
+            logging.error(f"AdHocBoost internals conform to the sklearn estimator API, so `stage_model_cls` must "
+                          f"subclass sklearn.base.BaseEstimator")
+            raise e
+
         # Set the attributes from input parameters.
         self.number_stages = number_stages
         self.stage_model_cls = stage_model_cls
         self.stage_model_hyper_param_sets = stage_model_hyper_param_sets
         self.stage_proba_thresholds = stage_proba_thresholds
-        self.sample_weight_multipliers = sample_weight_multipliers
+        self.stage_positive_sample_weights = stage_positive_sample_weights
         self.stage_models = [stage_model_cls(**hyper_param_set, objective="binary") for hyper_param_set in
                              self.stage_model_hyper_param_sets]
 
     @classmethod
-    def preprocess_config(cls, config_dict):
+    def preprocess_config(cls, config_dict) -> dict:
         """
         Function for working more easily with hyperparameter optimization. Takes in a flat dictionary of config, and
         maps to the necessary config structure for model creation.
@@ -114,13 +123,13 @@ class AdHocBoost:
         number_stages = get_number_stages(config_dict)
 
         # Create empty lists for storing the preprocessed parameters.
-        sample_weight_multiplier_list = []
+        stage_positive_sample_weights_list = []
         proba_thresholds_list = []
         model_hyperparameters_list = []
         for stage_number in range(number_stages):
 
             # Pop and store the model-agnostic hyperparams.
-            sample_weight_multiplier_list.append(config_dict.pop(f'model_{stage_number}__sample_weight_multiplier'))
+            stage_positive_sample_weights_list.append(config_dict.pop(f'model_{stage_number}__positive_sample_weight'))
             if stage_number < number_stages - 1:
                 proba_threshold = config_dict.pop(f"stage_{stage_number}__proba_threshold")
                 proba_thresholds_list.append(proba_threshold)
@@ -136,9 +145,9 @@ class AdHocBoost:
         return {'number_stages': number_stages,
                 'stage_model_hyper_param_sets': model_hyperparameters_list,
                 'stage_proba_thresholds': proba_thresholds_list,
-                'sample_weight_multipliers': sample_weight_multiplier_list}
+                'stage_positive_sample_weights': stage_positive_sample_weights_list}
 
-    def fit(self, train_X, train_y):
+    def fit(self, train_X, train_y) -> None:
         """
         Fit the model.
         :param train_X: The train feature-array, of shape (n-samples, n-features)
@@ -179,7 +188,7 @@ class AdHocBoost:
             # Fit the stage model.
             stage_sample_weight = self.get_sample_weight_column(
                 train_y[stage_mask],
-                positive_sample_multiplier=self.get_stage_sample_weight_multiplier(stage_number))
+                positive_sample_weight=self.get_stage_sample_weight(stage_number))
             self.fit_stage_model(
                 train_X[stage_mask],
                 train_y[stage_mask],
@@ -191,7 +200,10 @@ class AdHocBoost:
             stage_prediction = self.stage_predict_proba(train_X[stage_mask], stage_number)
             stage_prediction_df.loc[stage_mask, f"stage{stage_number}"] = stage_prediction
 
-    def fit_stage_model(self, stage_train_X, stage_train_y, stage_sample_weight, stage_number=0):
+        # log
+        logging.info(f"Fitting completed.")
+
+    def fit_stage_model(self, stage_train_X, stage_train_y, stage_sample_weight, stage_number=0) -> None:
         """
         Fits the stage_number'th model.
         :param stage_train_X: The train feature-array, of shape (n-samples, n-features)
@@ -212,33 +224,28 @@ class AdHocBoost:
             stage_train_y,
             sample_weight=stage_sample_weight)
 
-    def get_sample_weight_column(self, y, positive_sample_multiplier=1.7):
+    def get_sample_weight_column(self, y, positive_sample_weight=1) -> np.array:
         """
         Gets sample-weight column for use in fitting a model. The negative class is weighted as `1`, and the positive
         class is weighted as `positive_sample_multiplier * sqrt(|Positives| / |Negatives|)`.
+        :param positive_sample_weight:
         :param y: Array of labels.
         :type y: np.array
-        :param positive_sample_multiplier: The scalar constant for multiplying class weight.
-        :type positive_sample_multiplier: float
         :return:
         """
-        base_true_weight = np.sqrt(
-            (y == False).sum() /
-            (y == True).sum()
-        )
         sample_weight_column = np.ones(y.shape[0])
-        sample_weight_column[y == 1] = positive_sample_multiplier * base_true_weight
+        sample_weight_column[y == 1] = positive_sample_weight
         return sample_weight_column
 
-    def stage_predict_proba(self, data, stage_number):
+    def stage_predict_proba(self, data, stage_number) -> np.array:
         logging.info(f"Predicting proba on stage {stage_number} ({data.shape[0]} data points)...")
         return self.get_stage_model(stage_number).predict_proba(data)[:, 1]
 
-    def stage_predict(self, data, stage_number):
+    def stage_predict(self, data, stage_number) -> np.array:
         logging.info(f"Predicting label on stage {stage_number} ({data.shape[0]} data points)...")
         return self.get_stage_model(stage_number).predict(data)
 
-    def predict_proba(self, data, predict_labels=False):
+    def predict_proba(self, data, predict_labels=False) -> np.array:
         """
         Predict probabilities.
         :param data: A feature-array, of shape (n-samples, n-features)
@@ -249,9 +256,7 @@ class AdHocBoost:
         """
 
         # Before predicting, check if the model has been fitted
-        if not self.is_fitted():
-            logging.error(f"Model is not completely fitted; cannot execute prediction.")
-            raise NotFittedError
+        self.is_fitted()
 
         # Create a dataframe for tracking the prediction from each stage.
         logging.info(f"Predicting {'label' if predict_labels else 'proba'} from entire model pipeline:")
@@ -290,7 +295,7 @@ class AdHocBoost:
         predicted_probas = stage_prediction_df[f"stage{self.number_stages - 1}"].to_numpy().reshape((-1, 1))
         return np.hstack([np.zeros((predicted_probas.shape[0], 1)), predicted_probas])
 
-    def predict(self, data):
+    def predict(self, data) -> np.array:
         """
         Predict labels.
         :param data: A feature-array, of shape (n-samples, n-features)
@@ -299,22 +304,23 @@ class AdHocBoost:
         """
 
         # Before predicting, check if the model has been fitted
-        if not self.is_fitted():
-            logging.error(f"Model is not completely fitted; cannot execute prediction.")
-            raise NotFittedError
+        self.is_fitted()
 
         # return prediction
         return self.predict_proba(data, predict_labels=True)
 
-    def is_fitted(self) -> bool:
+    def is_fitted(self) -> None:
         """
-        Helper function to check if the model has been completely trained or not.
-        :return: bool
+        Helper function to check if the model has been completely trained or not. Returns None or raises an error.
+        :return: None
         """
-        for model in self.stage_models:
-            if not check_is_fitted(model):
-                return False
-        return True
+        try:
+            for i, model in enumerate(self.stage_models):
+                print(i, model)
+                print(check_is_fitted(model))
+        except NotFittedError as e:
+            logging.error(f"Model is not completely fitted; cannot execute prediction.")
+            raise e
 
     def get_stage_model(self, stage_number: int):
         """
@@ -325,16 +331,16 @@ class AdHocBoost:
         """
         return self.stage_models[stage_number]
 
-    def get_stage_sample_weight_multiplier(self, stage_number: int):
+    def get_stage_sample_weight(self, stage_number: int) -> float:
         """
         Getter helper function; gets and returns the stage_number'th sample_weight_multiplier.
         :param stage_number: Sample weight to retrieve
         :type stage_number: int
         :return: float
         """
-        return self.sample_weight_multipliers[stage_number]
+        return self.stage_positive_sample_weights[stage_number]
 
-    def get_stage_proba_threshold(self, stage_number: int):
+    def get_stage_proba_threshold(self, stage_number: int) -> float:
         """
         Getter helper function; gets and returns the stage_number'th probability threshold.
         :param stage_number: Model to retrieve
